import { Component, Input, Output, OnInit, NgZone, AfterViewInit, AfterContentChecked,
    AfterViewChecked, ElementRef, EventEmitter } from '@angular/core';
  import $ from 'jquery';
  import 'fullcalendar';
  import './lib/customEvent';
  import { ButtonClickModel } from './models/buttonClickModel';
  import { UpdateEventModel } from './models/updateEventModel';
  import { RenderEventModel } from './models/renderEventModel';
  import { EventObjectInput, OptionsInput } from 'fullcalendar/src/types/input-types';
  @Component({
      // tslint:disable-next-line
      selector: 'ng-fullcalendar',
      template: '',
  })
  export class CalendarComponent implements OnInit, AfterViewInit, AfterContentChecked, AfterViewChecked {
      private _eventsModel: EventObjectInput[];
      private _reRender = true;
      get eventsModel(): EventObjectInput[] {
          return this._eventsModel;
      }
  
      @Input('eventsModel')
      set eventsModel(value: EventObjectInput[]) {
          this._eventsModel = value;
          if (this._reRender) {
              setTimeout(() => {
                  this.renderEvents(value);
              }, 50);
          } else {
              this._reRender = true;
          }
      }
      @Output()
      eventsModelChange = new EventEmitter<any>();
  
      @Input() options: OptionsInput;
      @Output() eventDrop = new EventEmitter<any>();
      @Output() eventResize = new EventEmitter<any>();
      @Output() eventClick = new EventEmitter<any>();
      @Output() clickButton = new EventEmitter<any>();
      @Output() windowResize = new EventEmitter<any>();
      @Output() viewRender = new EventEmitter<any>();
      @Output() viewDestroy = new EventEmitter<any>();
      @Output() eventRender = new EventEmitter<any>();
      @Output() initialized = new EventEmitter<any>();
      @Output() select = new EventEmitter<any>();
      @Output() unselect = new EventEmitter<any>();
      @Output() dayClick = new EventEmitter<any>();
      @Output() navLinkDayClick = new EventEmitter<any>();
      @Output() navLinkWeekClick = new EventEmitter<any>();
  
      constructor(private element: ElementRef, private zone: NgZone) {
      }
  
      ngOnInit(): void {
      }
  
      ngAfterViewInit() {
          setTimeout(() => {
              this.updaterOptions();
              this.zone.runOutsideAngular(() => {
                  $(this.element.nativeElement).fullCalendar(this.options);
                  this._eventsModel = this.options.events as EventObjectInput[];
                  this.eventsModelChange.next(this.options.events);
                  this.initialized.emit(true);
                  // Click listeners
                  let elem = document.getElementsByTagName('ng-fullcalendar');
  
                  $('[class ^="fc"][class *="button"]').click(el => {
                      let classnames = el.currentTarget.className.split(' ');
                      classnames.forEach(name => {
                          if (name.indexOf('button') === name.length - 6) {
                              name = name.replace(/fc|button|-/g, '');
                              if (name !== '') {
                                  this.renderEvents(this._eventsModel);
                                  eventDispatch(name);
                              }
                          }
                      });
                  });
                  function eventDispatch(buttonType: string) {
                      let data = $('ng-fullcalendar').fullCalendar('getDate');
                      let currentDetail: ButtonClickModel = {
                          buttonType: buttonType,
                          data: data
                      };
                      let widgetEvent = new CustomEvent('clickButton', {
                          bubbles: true,
                          detail: currentDetail
                      });
                      for (let i = 0; i < elem.length; i++) {
                          elem[i].dispatchEvent(widgetEvent);
                      }
                  }
              });
          }, );
  
      }
      ngAfterContentChecked() {
      }
      ngAfterViewChecked() {
      }
      updateEventsBeforeResize() {
          let events = this.fullCalendar('clientEvents');
          this._reRender = false;
          this.eventsModel = events;
          this.eventsModelChange.next(events);
      }
      updaterOptions() {
          let elem = document.getElementsByTagName('ng-fullcalendar');
          this.options.eventDrop = (event, duration) => {
              let detail: UpdateEventModel = { event: event, duration: duration };
              let widgetEvent = new CustomEvent('eventDrop', {
                  bubbles: true,
                  detail: detail
              });
              this.updateEventsBeforeResize();
              for (let i = 0; i < elem.length; i++) {
                  elem[i].dispatchEvent(widgetEvent);
              }
          };
          this.options.eventResize = (event, duration) => {
              let detail: UpdateEventModel = { event: event, duration: duration };
              let widgetEvent = new CustomEvent('eventResize', {
                  bubbles: true,
                  detail: detail
              });
              this.updateEventsBeforeResize();
              for (let i = 0; i < elem.length; i++) {
                  elem[i].dispatchEvent(widgetEvent);
              }
          };
          this.options.eventRender = function (event, element, view) {
              let detail: RenderEventModel = { event: event, element: element, view: view };
            let widgetEvent = new CustomEvent('eventRender', {
                  bubbles: true,
                  detail: detail
              });
              for (let i = 0; i < elem.length; i++) {
                  elem[i].dispatchEvent(widgetEvent);
              }
          };
          this.options.eventClick = (event) => {
              let detail: UpdateEventModel = { event: event, duration: null };
              let widgetEvent = new CustomEvent('eventClick', {
                  bubbles: true,
                  detail: detail
              });
              for (let i = 0; i < elem.length; i++) {
                  elem[i].dispatchEvent(widgetEvent);
              }
  
          };
          this.options.windowResize = function (view) {
              let detail = { view: view };
              let widgetEvent = new CustomEvent('windowResize', {
                  bubbles: true,
                  detail: detail
              });
              if (elem && elem[0]) {
                  for (let i = 0; i < elem.length; i++) {
                      elem[i].dispatchEvent(widgetEvent);
                  }
              }
          };
          this.options.viewRender = function (view, element) {
              let detail = { view: view, element: element };
              let widgetEvent = new CustomEvent('viewRender', {
                  bubbles: true,
                  detail: detail
              });
              for (let i = 0; i < elem.length; i++) {
                  elem[i].dispatchEvent(widgetEvent);
              }
          };
          this.options.viewDestroy = function (view, element) {
              let detail = { view: view, element: element };
              let widgetEvent = new CustomEvent('viewDestroy', {
                  bubbles: true,
                  detail: detail
              });
              for (let i = 0; i < elem.length; i++) {
                  elem[i].dispatchEvent(widgetEvent);
              }
          };
          this.options.select = function (start: any, end: any, jsEvent: MouseEvent, view: any, resource?: any) {
              let detail = { start: start, end: end, jsEvent: jsEvent, view: view, resource: resource };
              let widgetEvent = new CustomEvent('select', {
                  bubbles: true,
                  detail: detail
              });
              for (let i = 0; i < elem.length; i++) {
                  elem[i].dispatchEvent(widgetEvent);
              }
          };
          this.options.unselect = function (view: any, jsEvent: Event) {
              let detail = { view: view, jsEvent: jsEvent };
              let widgetEvent = new CustomEvent('unselect', {
                  bubbles: true,
                  detail: detail
              });
              for (let i = 0; i < elem.length; i++) {
                  elem[i].dispatchEvent(widgetEvent);
              }
          };
          this.options.dayClick = function (date: any, jsEvent: Event, view: any) {
              let detail = { date: date, jsEvent: jsEvent, view: view };
              let widgetEvent = new CustomEvent('dayClick', {
                  bubbles: true,
                  detail: detail
              });
              for (let i = 0; i < elem.length; i++) {
                  elem[i].dispatchEvent(widgetEvent);
              }
          };
          this.options.navLinkDayClick = function (date: any, jsEvent: Event) {
              let detail = { date: date, jsEvent: jsEvent };
              let widgetEvent = new CustomEvent('navLinkDayClick', {
                  bubbles: true,
                  detail: detail
              });
              for (let i = 0; i < elem.length; i++) {
                  elem[i].dispatchEvent(widgetEvent);
              }
          };
          this.options.navLinkWeekClick = function (weekStart: any, jsEvent: Event) {
              let detail = { weekStart: weekStart, jsEvent: jsEvent };
              let widgetEvent = new CustomEvent('navLinkWeekClick', {
                  bubbles: true,
                  detail: detail
              });
              for (let i = 0; i < elem.length; i++) {
                  elem[i].dispatchEvent(widgetEvent);
              }
          };
      }
  
      fullCalendar(...args: any[]): any {
          if (!args) {
              return;
          }
          switch (args.length) {
              case 0:
                  return;
              case 1:
                  return $(this.element.nativeElement).fullCalendar(args[0]);
              case 2:
                  return $(this.element.nativeElement).fullCalendar(args[0], args[1]);
              case 3:
                  return $(this.element.nativeElement).fullCalendar(args[0], args[1], args[2]);
          }
      }
  
      updateEvent(event: any) {
          return $(this.element.nativeElement).fullCalendar('updateEvent', event);
      }
  
      clientEvents(idOrFilter: any): any {
          return $(this.element.nativeElement).fullCalendar('clientEvents', idOrFilter);
      }
  
      renderEvents(events: EventObjectInput[]) {
          $(this.element.nativeElement).fullCalendar('removeEvents');
          if (events && events.length > 0) {
              $(this.element.nativeElement).fullCalendar('renderEvents', events, true);
              $(this.element.nativeElement).fullCalendar('rerenderEvents');
          }
      }
  
  }